@@ -30,6 +30,8 @@
 
 G_BEGIN_DECLS
 
+#define HILDON_DESKTOP_ITEM_SETTINGS_SYMBOL "hildon_desktop_item_get_settings"
+
 typedef struct _HildonDesktopItem HildonDesktopItem; 
 typedef struct _HildonDesktopItemClass HildonDesktopItemClass;
 
@@ -70,6 +72,8 @@ const gchar*  hildon_desktop_item_get_name    (HildonDesktopItem *item);
 gint          hildon_desktop_item_find_by_id  (HildonDesktopItem *item,
                                                const gchar *id);
 
+typedef GtkWidget  *(*HildonDesktopItemSettingsDialog) (void);
+
 G_END_DECLS
 
 #endif
