@@ -34,10 +34,13 @@
 #include <X11/extensions/Xcomposite.h>
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xrender.h>
+
+#include <libhildondesktop/hildon-desktop-picture.h>
 #endif
 
 #include <errno.h>
-#include <string.h> /* malloc */
+#include <string.h>
+#include <stdlib.h> /* malloc */
 
 enum
 {
@@ -64,7 +67,11 @@ typedef struct
   Picture       alpha_mask;
   Picture       alpha_mask_unscaled;
   gint          background_width, background_height;
-  gulong        realize_handler, style_set_handler, size_allocate_handler;
+  gint          state;
+  gulong        realize_handler,
+                style_set_handler,
+                size_allocate_handler,
+                state_change_handler;
 #endif
 } ChildData;
 
@@ -201,6 +208,9 @@ static void
 hildon_home_area_child_realize (GtkWidget             *child,
                                 HildonHomeArea        *area);
 
+static void
+hildon_home_area_child_state_change (ChildData         *child_data);
+
 #endif
 
 
@@ -750,6 +760,7 @@ hildon_home_area_allocate_child (HildonHomeArea        *area,
       border_width = GTK_CONTAINER (area)->border_width;
 
       gtk_widget_get_child_requisition (child->widget, &child_requisition);
+
       child_allocation.x = widget->allocation.x + child->x + border_width;
       child_allocation.y = widget->allocation.y + child->y + border_width;
 
@@ -958,8 +969,7 @@ hildon_home_area_forall (GtkContainer  *container,
 #ifdef HAVE_X_COMPOSITE
 static void
 hildon_home_area_child_compose (ChildData *child_data,
-                                HildonHomeArea *area,
-                                gboolean alpha)
+                                HildonHomeArea *area)
 {
   HildonHomeAreaPriv   *priv;
   GtkAllocation        *alloc;
@@ -972,9 +982,9 @@ hildon_home_area_child_compose (ChildData *child_data,
   alloc = &(child_data->widget->allocation);
 
   XRenderComposite (GDK_DISPLAY (),
-                    (alpha)?PictOpOver:PictOpSrc,
+                    (child_data->state == 0)?PictOpOver:PictOpSrc,
                     child_data->picture,
-                    (alpha)?child_data->alpha_mask:None,
+                    (child_data->state == 0)?child_data->alpha_mask:None,
                     priv->picture,
                     0,
                     0,
@@ -1043,15 +1053,7 @@ hildon_home_area_child_build_alpha_mask_unscaled (HildonHomeArea       *area,
                                                   GtkWidget            *child)
 {
   HildonHomeAreaPriv           *priv;
-  GdkPixbuf                    *pixbuf = NULL;
-  GC                            gc;
-  XGCValues                     gc_values;
-  XRenderPictureAttributes      pa;
-  XRenderPictFormat            *format;
-  XImage                       *image;
-  Pixmap                        pixmap;
   ChildData                    *child_data;
-  GError                       *error = NULL;
   const gchar                  *mask_file_name = NULL;
 
   priv = HILDON_HOME_AREA_GET_PRIVATE (area);
@@ -1071,116 +1073,14 @@ hildon_home_area_child_build_alpha_mask_unscaled (HildonHomeArea       *area,
 
   mask_file_name = child->style->rc_style->bg_pixmap_name[GTK_STATE_PRELIGHT];
 
-  if (mask_file_name)
-    {
-      pixbuf = gdk_pixbuf_new_from_file (mask_file_name,
-                                         &error);
-    }
+  if (!mask_file_name)
+    return;
 
-  if (error)
-    {
-      g_warning ("Could not open alpha mask from style: %s", error->message);
-      g_error_free (error);
-
-      pixbuf = NULL;
-    }
-
-  if (pixbuf)
-    {
-      gchar                        *data;
-      guchar                       *pixbuf_data;
-      guint                         i;
-      gint                          pw, ph;
-
-      child_data->background_width = pw = gdk_pixbuf_get_width (pixbuf);
-      child_data->background_height = ph = gdk_pixbuf_get_height (pixbuf);
-
-      pixmap = XCreatePixmap (GDK_DISPLAY (),
-                              GDK_WINDOW_XID (GTK_WIDGET (area)->window),
-                              pw,
-                              ph,
-                              8);
-
-      /* Use malloc here because it is freed by Xlib */
-      data = (gchar *) malloc (pw*ph);
-
-      image = XCreateImage (GDK_DISPLAY (),
-                            None,
-                            8,         /* depth */
-                            ZPixmap,
-                            0,         /* offset */
-                            data,
-                            pw,
-                            ph,
-                            8,
-                            pw);
-
-      pixbuf_data = gdk_pixbuf_get_pixels (pixbuf);
-
-      for (i = 0; i < pw * ph; i++)
-        {
-          data[i] = pixbuf_data[4*i + 3];
-        }
-
-      gc = XCreateGC (GDK_DISPLAY (),
-                      pixmap,
-                      0,
-                      &gc_values);
-
-      XPutImage (GDK_DISPLAY (),
-                 pixmap,
-                 gc,
-                 image,
-                 0, 0,
-                 0, 0,
-                 pw, ph);
-
-      XFreeGC (GDK_DISPLAY (), gc);
-      XDestroyImage (image);
-      g_object_unref (pixbuf);
-    }
-  else
-    {
-      pixmap = XCreatePixmap (GDK_DISPLAY (),
-                              GDK_WINDOW_XID (GTK_WIDGET (area)->window),
-                              1,
-                              1,
-                              8);
-    }
-
-  if (pixmap == None)
-    {
-      g_warning ("Could not create pixmap for alpha_channel");
-      return;
-    }
-
-  format = XRenderFindStandardFormat (GDK_DISPLAY (),
-                                      PictStandardA8);
-
-  pa.repeat = True;
-  child_data->alpha_mask_unscaled = XRenderCreatePicture (GDK_DISPLAY (),
-                                                          pixmap,
-                                                          format,
-                                                          CPRepeat,
-                                                          &pa);
-
-  if (!pixbuf)
-    {
-      XRenderColor c = {0};
-
-/*      c.alpha = priv->default_alpha * 0xffff;*/
-      c.alpha = 0xFFFF;
-
-      XRenderFillRectangle (GDK_DISPLAY (),
-                            PictOpSrc,
-                            child_data->alpha_mask_unscaled,
-                            &c,
-                            0, 0, 1, 1);
-
-      child_data->alpha_mask_unscaled = None;
-    }
-
-  XFreePixmap (GDK_DISPLAY (), pixmap);
+  hildon_desktop_picture_and_mask_from_file (mask_file_name,
+                                             NULL,
+                                             &child_data->alpha_mask_unscaled,
+                                             &child_data->background_width,
+                                             &child_data->background_height);
 
 }
 
@@ -1195,10 +1095,6 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
   if (!GTK_WIDGET_REALIZED (child))
     return;
 
-  gtk_widget_style_get (child,
-                        "background-borders", &borders,
-                        NULL);
-
   gtk_container_child_get (GTK_CONTAINER (area), child,
                            "child-data", &child_data,
                            NULL);
@@ -1215,6 +1111,10 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
 
   child_data->alpha_mask = None;
 
+  gtk_widget_style_get (child,
+                        "background-borders", &borders,
+                        NULL);
+
   if (borders && child_data->alpha_mask_unscaled)
     {
       Pixmap                    p;
@@ -1227,7 +1127,10 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
       h = child->allocation.height;
 
       if (w < 0 || h < 0)
-        return;
+        {
+          g_free (borders);
+          return;
+        }
 
       center_w = w - borders->left - borders->right;
       center_h = h - borders->top  - borders->bottom;
@@ -1238,6 +1141,7 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
       if (scenter_h < 0 || scenter_w < 0)
         {
           g_warning ("Invalid border values, alpha mask will not be applied");
+          g_free (borders);
           return;
         }
 
@@ -1386,24 +1290,28 @@ hildon_home_area_child_build_alpha_mask (HildonHomeArea *area,
 
 }
 
+static void
+hildon_home_area_child_state_change (ChildData *child_data)
+{
+  gint state;
+
+  g_object_get (child_data->widget,
+                "state", &state,
+                NULL);
+
+  g_debug ("state changed to %i", state);
+
+  child_data->state = state;
+
+}
+
+
 static gint
 find_by_window (ChildData *data, Window *w)
 {
   return !(data->window == *w);
 }
 
-static void
-child_is_dragged (ChildData *child, gboolean *is_dragged)
-{
-  gint  state;
-
-  g_object_get (child->widget,
-                "state", &state,
-                NULL);
-  if (!*is_dragged && state)
-    *is_dragged = TRUE;
-}
-
 static gboolean
 hildon_home_area_expose (GtkWidget *widget,
                          GdkEventExpose *event)
@@ -1417,11 +1325,8 @@ hildon_home_area_expose (GtkWidget *widget,
       XRectangle                rectangle;
       XserverRegion             region;
       GdkDrawable              *drawable;
-      XRenderPictFormat        *format;
-      XRenderPictureAttributes  pa;
       Window                   *wchildren, root, parent;
       int                       n_children, i;
-      gboolean                  is_dragged = FALSE;
 
       gdk_window_get_internal_paint_info (widget->window,
                                           &drawable,
@@ -1433,16 +1338,7 @@ hildon_home_area_expose (GtkWidget *widget,
       rectangle.width = event->area.width;
       rectangle.height = event->area.height;
 
-      format = XRenderFindVisualFormat (GDK_DISPLAY(),
-                                        GDK_VISUAL_XVISUAL (gdk_drawable_get_visual (drawable)));
-
-      pa.subwindow_mode = IncludeInferiors;
-
-      priv->picture = XRenderCreatePicture (GDK_DISPLAY (),
-                                            GDK_DRAWABLE_XID (drawable),
-                                            format,
-                                            CPSubwindowMode,
-                                            &pa);
+      priv->picture = hildon_desktop_picture_from_drawable (drawable);
 
       region = XFixesCreateRegion (GDK_DISPLAY (),
                                    &rectangle,
@@ -1473,10 +1369,6 @@ hildon_home_area_expose (GtkWidget *widget,
           return FALSE;
         }
 
-      g_list_foreach (priv->children_data,
-                      (GFunc)child_is_dragged,
-                      &is_dragged);
-
       for (i = 0; i < n_children; i++)
         {
           GList        *l = NULL;
@@ -1487,8 +1379,7 @@ hildon_home_area_expose (GtkWidget *widget,
 
           if (l)
             hildon_home_area_child_compose (l->data,
-                                            HILDON_HOME_AREA (widget),
-                                            !is_dragged);
+                                            HILDON_HOME_AREA (widget));
         }
 
       XFree (wchildren);
@@ -1910,6 +1801,8 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
                                            child_data->style_set_handler);
               g_signal_handler_disconnect (applet,
                                            child_data->size_allocate_handler);
+              g_signal_handler_disconnect (applet,
+                                           child_data->state_change_handler);
             }
 #endif
 
@@ -1917,9 +1810,6 @@ hildon_home_area_remove (GtkContainer *area, GtkWidget *applet)
 
           child_data_free (child_data);
         }
-
-  if (GTK_CONTAINER_CLASS (parent_class)->remove)
-    GTK_CONTAINER_CLASS (parent_class)->remove (area, applet);
 }
 
 /* Public functions */
@@ -2243,17 +2133,25 @@ hildon_home_area_set_batch_add (HildonHomeArea *area, gboolean batch_add)
 void
 hildon_home_area_put (HildonHomeArea *area, GtkWidget *widget, gint x, gint y)
 {
-  ChildData  *child_data;
+  ChildData    *child_data;
+  gint          state = 0;
 
   g_return_if_fail (HILDON_IS_HOME_AREA (area) && GTK_IS_WIDGET (widget));
 
   gtk_widget_set_parent (widget, GTK_WIDGET (area));
 
+  if (HILDON_DESKTOP_IS_HOME_ITEM (widget))
+    g_object_get (widget,
+                  "state", &state,
+                  NULL);
+
   child_data = g_new0 (ChildData, 1);
   child_data->widget = widget;
   child_data->x = x;
   child_data->y = y;
-
+#ifdef HAVE_X_COMPOSITE  
+  child_data->state = state;
+#endif
   gtk_container_child_set (GTK_CONTAINER (area), widget,
                            "child-data", child_data,
                            NULL);
@@ -2274,6 +2172,15 @@ hildon_home_area_put (HildonHomeArea *area, GtkWidget *widget, gint x, gint y)
                             G_CALLBACK (hildon_home_area_child_size_allocate),
                             area);
 
+      if (HILDON_DESKTOP_IS_HOME_ITEM (widget))
+        child_data->state_change_handler =
+            g_signal_connect_swapped (widget, "notify::state",
+                                      G_CALLBACK (
+                                         hildon_home_area_child_state_change),
+                                      child_data);
+
+
+
       if (GTK_WIDGET_REALIZED (widget))
         {
           hildon_home_area_create_child_data (HILDON_HOME_AREA (area), widget);
