@@ -281,6 +281,8 @@ hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
   {
     hildon_desktop_popup_window_popdown (window);
 
+    switcher->priv->is_open = FALSE;
+
     if (event->keyval == GDK_Escape)
     {
       /* pass focus to the last active application */
@@ -290,8 +292,8 @@ hd_switcher_menu_popup_window_keypress_cb (GtkWidget      *widget,
     {
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), FALSE);
 
-      GdkWindow *window = gtk_widget_get_parent_window (GTK_WIDGET (switcher));
-      gtk_widget_grab_focus (GTK_WIDGET (switcher));
+      GdkWindow *window = gtk_widget_get_parent_window (switcher->priv->toggle_button);
+      gtk_widget_grab_focus (GTK_WIDGET (switcher->priv->toggle_button));
       hd_wm_activate_window (HD_TN_ACTIVATE_KEY_FOCUS,window);
     }
 
@@ -428,6 +430,9 @@ hd_switcher_menu_constructor (GType gtype,
   switcher->priv->popup_window =
     HILDON_DESKTOP_POPUP_WINDOW 
       (hildon_desktop_popup_window_new (1,GTK_ORIENTATION_HORIZONTAL,HD_POPUP_WINDOW_DIRECTION_RIGHT_BOTTOM));	     
+
+  hildon_desktop_popup_window_attach_widget
+    (switcher->priv->popup_window, button);	  
   
   switcher->priv->notifications_window = 
     hildon_desktop_popup_window_get_pane (switcher->priv->popup_window, 0);
@@ -741,6 +746,8 @@ hd_switcher_menu_item_activated (GtkMenuItem *menuitem, HDSwitcherMenu *switcher
     hildon_desktop_popup_window_popdown
       (switcher->priv->popup_window);
 
+  switcher->priv->is_open = FALSE;
+  
   gtk_toggle_button_set_active
     (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), FALSE);
 }	
@@ -786,9 +793,13 @@ hd_switcher_menu_check_content (HDSwitcherMenu *switcher)
   apps = hd_wm_get_applications (switcher->hdwm);
   
   if (apps || children)
-  {
+  {    
+    g_object_set (switcher->priv->toggle_button, "can-focus", TRUE, NULL);
+    gtk_widget_grab_focus (switcher->priv->toggle_button);
+    gtk_widget_set_sensitive (switcher->priv->toggle_button, TRUE);
+
     gtk_widget_show (GTK_BIN (switcher->priv->toggle_button)->child);
-     
+
     if (children)
     {	     
       if (GTK_BIN (switcher->priv->toggle_button)->child != switcher->priv->icon)
@@ -822,11 +833,17 @@ hd_switcher_menu_check_content (HDSwitcherMenu *switcher)
      hd_switcher_menu_update_highlighting (switcher, FALSE);
 	  
      gtk_widget_hide (GTK_BIN (switcher->priv->toggle_button)->child);	  
-
+     g_object_set (switcher->priv->toggle_button, "can-focus", FALSE, NULL);
+     gtk_widget_set_sensitive (switcher->priv->toggle_button, FALSE);
+     
      if (switcher->priv->is_open)
+     {	     
        hildon_desktop_popup_window_popdown 
          (switcher->priv->popup_window);	       
-
+ 
+       switcher->priv->is_open = FALSE;
+     }
+     
      gtk_toggle_button_set_active 
        (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button), FALSE);
   }
@@ -994,15 +1011,22 @@ hd_switcher_menu_toggled_cb (GtkWidget *button, HDSwitcherMenu *switcher)
   if (!gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (switcher->priv->toggle_button)))
   {
     hildon_desktop_popup_window_popdown (switcher->priv->popup_window);
+    switcher->priv->is_open = FALSE;
     return;
-  }    
-	
+  }   
+
+  if (switcher->priv->is_open)
+  {	  
+    hildon_desktop_popup_menu_select_next_item (switcher->priv->menu_applications);
+    g_debug ("selecting next item");
+  }
+    
   hildon_desktop_popup_window_popup 
    (switcher->priv->popup_window,
     hd_switcher_menu_position_func,
     (gpointer)switcher,
     GDK_CURRENT_TIME);
-  
+
   if (!hildon_desktop_popup_menu_get_children (switcher->priv->menu_notifications))
     gtk_widget_hide (hildon_desktop_popup_window_get_pane (switcher->priv->popup_window, 1));
   else
@@ -1365,6 +1389,7 @@ hd_switcher_menu_changed_stack_cb (HDWM *hdwm,
           (switcher->priv->menu_applications, GTK_MENU_ITEM (l->data));
 	
 	switcher->priv->active_menu_item = GTK_WIDGET (l->data);
+	break;
       }
     }	    
   }	  
@@ -1448,7 +1473,8 @@ hd_switcher_menu_long_press_cb (HDWM *hdwm, HDSwitcherMenu *switcher)
   {
     hildon_desktop_popup_menu_activate_item 
       (switcher->priv->menu_applications, 
-       GTK_MENU_ITEM (switcher->priv->active_menu_item));
+       hildon_desktop_popup_menu_get_selected_item 
+         (switcher->priv->menu_applications));
   }
   else
     hd_wm_top_desktop ();
@@ -1462,6 +1488,9 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
 {
   GdkPixbuf *icon = NULL;
   gchar *summary = NULL, *body = NULL;
+  const gchar *category = NULL;
+  GHashTable *hints;
+  GValue *hint;
   guint id;
   GtkWidget *menu_item;
  
@@ -1478,8 +1507,19 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
 		      HD_NM_COL_ICON, &icon,
 		      HD_NM_COL_SUMMARY, &summary,
 		      HD_NM_COL_BODY, &body,
+		      HD_NM_COL_HINTS, &hints,
 		      -1);
 
+  hint = g_hash_table_lookup (hints, "category");
+
+  if (hint)
+  {	  
+    category = g_value_get_string (hint);
+
+    if (g_str_equal (category, "system.note.dialog"))
+      goto out;	  
+  }
+  
   if (switcher->priv->last_iter_added == NULL)
     hd_switcher_menu_notification_deleted_cb 
       (HILDON_DESKTOP_NOTIFICATION_MANAGER (tree_model), id, switcher);	    
@@ -1499,10 +1539,13 @@ hd_switcher_menu_notification_changed_cb (GtkTreeModel   *tree_model,
   hildon_desktop_popup_menu_add_item
     (switcher->priv->menu_notifications, 
      GTK_MENU_ITEM (gtk_separator_menu_item_new ()));	 
-    	    
+out:    	    
   switcher->priv->last_iter_added = NULL;
 
   hd_switcher_menu_check_content (switcher);
+
+  g_free (summary);
+  g_free (body);
 }
 
 static void 
